#include "stdafx.h"
#include <JAMA_SVD.h>

const double CONSTR_XOFFSET =  2.0;
const double CONSTR_YOFFSET = 10.0;
//---- 
const COLORREF g_aClassColors[10] = { 
	RGB( 0, 0, 0), RGB( 255, 0, 0 ), 
	RGB( 0, 0, 128), RGB( 0, 128, 0), 
	RGB( 255, 255, 0), RGB( 255, 0, 255), 
	RGB( 0, 255, 0), RGB( 0, 0, 255), 
	RGB( 0, 255, 255), RGB( 0, 128, 255)
	};
COLORREF DefaultClassColor( int iClass ) { 
	if( iClass<10 ) 
		return g_aClassColors[iClass];
	return RGB(16*(iClass%16),40*(iClass%7),85*(iClass%4));
	}	

//-- calc endpoint of arrowhead on rate from pt0 to pt1
void CalcArrowPoint( POINT pt0, POINT pt1, POINT * pA, bool bNeg ) { 
	//-- strange calc for arrow endpoint, but it requires no floating 
	// point math or trigonometric functions.
	// From line pt0:pt1 turn left 90 degrees, go same distance, find 
	// pta=midpoint back toward pt0, draw along pt1:pta minimum 9 pixels 
	// of deltaX or deltaY
	if( ! bNeg ) {
		pA->x = (pt0.x+pt1.x+(pt0.y-pt1.y))/2;
		pA->y = (pt0.y+pt1.y+(pt1.x-pt0.x))/2;
		}
	else {
		pA->x = (pt0.x+pt1.x+(pt1.y-pt0.y))/2;
		pA->y = (pt0.y+pt1.y+(pt0.x-pt1.x))/2;
		}
		
	if( abs(pA->y-pt1.y) > abs(pA->x-pt1.x) ) { 
		pA->x=pt1.x+MulDiv(pA->x-pt1.x,9,abs(pA->y-pt1.y));
		pA->y=pt1.y-(pA->y>pt1.y?-9:9);
		}
	else {
		pA->y=pt1.y+MulDiv(pA->y-pt1.y,9,abs(pA->x-pt1.x) );
		pA->x=pt1.x+(pA->x>pt1.x?9:-9);
		}
	}

//-----------------------------------------------------------------------------
CCombState :: CCombState( int iChannels, int iStates ) {
	m_ai = new int[m_iStates=iStates];
	for( int i=1; i<m_iStates; ++i ) 
		m_ai[i]=0;
	t=m_ai[0]=m_iChannels=iChannels;
	h=0;
	}

bool CCombState :: NextCombination( ) { 
	if( m_ai[m_iStates-1]==m_iChannels ) 
		return false;
	//-- Go to next combination 	
	h = t>1 ? 1 : h+1; 
	t=m_ai[h-1];
	m_ai[h-1]=0;
	m_ai[0]=t-1;
	m_ai[h]++;
	return true;
	}

CCombState :: ~CCombState() { 
	delete [] m_ai;
	}
	
//-----------------------------------------------------------------------------
CStimulus :: CStimulus( const char * pchName, double dValue ) { 
	m_cName = pchName;
	m_dValue = dValue;
	}
	
void CStimuli :: AddStimulus( const char * pchName, double dValue ) { 
	m_aStim.AddItem( new CStimulus( pchName, dValue ) );
	}
	
int CStimuli :: StimulusIndex( const char * pchName ) { 
	int iStim, iStims=m_aStim.Count();
	for( iStim=0; iStim<iStims; ++iStim ) 
		if( strcmp(((CStimulus *)m_aStim[iStim])->m_cName.Buffer(), pchName)==0 ) 
			return iStim;
	return -1;
	}

double CStimuli :: StimulusValue( const char * pchName, double t, double dNotFoundResult ) { 
	int iStim, iStims=m_aStim.Count();
	for( iStim=0; iStim<iStims; ++iStim ) 
		if( strcmp(((CStimulus *)m_aStim[iStim])->m_cName.Buffer(), pchName)==0 ) 
			return ((CStimulus *)m_aStim[iStim])->Value( t );
	return dNotFoundResult;
	}

CStimulus * CStimuli :: Stimulus( const char * pchName ) { 
	int iStim = StimulusIndex( pchName ) ;
	if( iStim == -1 ) 
		return NULL;
	return (CStimulus *)m_aStim[iStim];
	}

//-----------------------------------------------------------------------------
/*
CRateListItem :: CRateListItem( int iState0, int iState1 ) { 
	m_aiState[0]=iState0;
	m_aiState[1]=iState1;
	}
	
void CRateList :: AddRate( int iState0, int iState1 ) { 
	m_aRows.AddItem( new CRateListItem( iState0, iState1 ) );
	}

bool CRateList :: HasRate( int iState0, int iState1 ) { 
	int * aiStates;
	for( int iRow=0; iRow<m_aRows.Count(); ++iRow ) { 
		aiStates = ((CRateListItem  *)m_aRows[iRow])->m_aiState;
		if( ( aiStates[0]==iState0 && aiStates[1]==iState1 ) 
				|| ( aiStates[0]==iState1 && aiStates[1]==iState0 ) )
			return true;
		}
	return false;
	}
*/

//-----------------------------------------------------------------------------
CModelClass :: CModelClass() { 
	m_dAmp=1.0;
	m_dStdDev=0.1;
	m_dConditioning=1.0;
	}

CModelClass :: CModelClass( double dAmp, double dStd, double dCond, COLORREF cr ) { 
	m_dAmp=dAmp;
	m_dStdDev=dStd;
	m_dConditioning=dCond;
	m_cr=cr;
	}

//-----------------------------------------------------------------------------
/*
CModelState :: CModelState() { 
	m_dStartProb=0.0;
	m_iClass=0;
	m_dXPos=50.0;
	m_dEnergyMin=m_dEnergyMax=0.0;
	m_dYPos=50.0;
	m_iGroup=0;
	}
*/	
CModelState :: CModelState( int iClass, double dX, double dY, double dStartProb ) { 
	m_dStartProb=dStartProb;
	m_iClass=iClass;
	m_dXPos=dX;
	m_dEnergyMin=m_dEnergyMax=0.0;
	m_dYPos=dY;
	m_iGroup=0;
	}

double CModelState :: Y( double dFrac ) { 
	return m_dEnergyMin*(1-dFrac)+m_dEnergyMax*dFrac;
	}
	
//-----------------------------------------------------------------------------
CModelRate :: CModelRate( CModelRate * pRef ) {
	m_aiStates[0]=pRef->m_aiStates[0];
	m_adK0[0]=pRef->m_adK0[0];
	m_adK1[0]=pRef->m_adK1[0];
	m_abPDep[0]=pRef->m_abPDep[0];

	m_acPName[0].Set( pRef->m_acPName[0].ConstBuffer() );
	m_abQDep[0]=pRef->m_abQDep[0];
	m_acQName[0].Set( pRef->m_acQName[0].ConstBuffer() );
	m_aiStates[0]=pRef->m_aiStates[0];
	m_adK0[1]=pRef->m_adK0[1];
	m_adK1[1]=pRef->m_adK1[1];
	m_abPDep[1]=pRef->m_abPDep[1];
	m_acPName[1].Set( pRef->m_acPName[1].ConstBuffer() );
	m_abQDep[1]=pRef->m_abQDep[1];
	m_acQName[1].Set( pRef->m_acQName[1].ConstBuffer() );
	}

CModelRate :: CModelRate( int iSt0, int iSt1, double dK0Fd, double dK0Bk ) { 
	m_aiStates[0]=iSt0;
	m_aiStates[1]=iSt1;
	m_adK0[0]=dK0Fd;
	m_adK0[1]=dK0Bk;
	m_adK1[0]=0.0;
	m_adK1[1]=0.0;
	m_abPDep[0]=m_abPDep[1]=false;
	m_acPName[0].Set("Ligand");
	m_acPName[1].Set("Ligand");
	m_abQDep[0]=m_abQDep[1]=false;
	m_acQName[0].Set("Voltage");
	m_acQName[1].Set("Voltage");
	}
	
double CModelRate :: Y( double dFrac ) { 
	return m_dEnergyMin*(1-dFrac)+m_dEnergyMax*dFrac;
	}

int CModelRate :: IsHit( int iX, int iY, POINT pt0, POINT pt1, int iTextHgt, int iStateRadius ) { 
	POINT ptMin, ptMax;
	//-- Check if clicked near rate line 
	ptMin.x=min(pt0.x, pt1.x);
	ptMin.y=min(pt0.y, pt1.y);
	ptMax.x=max(pt0.x, pt1.x);
	ptMax.y=max(pt0.y, pt1.y);
	if(  ((ptMax.y-ptMin.y)>(ptMax.x-ptMin.x) ? iY>=ptMin.y+iStateRadius && iY<=ptMax.y-iStateRadius : iX>=ptMin.x+iStateRadius && iX<=ptMax.x-iStateRadius )
		&& DistanceFromPointToLine( iX, iY, pt0.x, pt0.y, pt1.x, pt1.y )<iTextHgt 
		)
		return 1;
	return 0;		
	}

void CModelRate :: SetLimits( double dFactor ) { 
	if( dFactor==0.0 ) 
		dFactor=1.0e-5;
	m_adK0Limit[0][0]=m_adK0[0]/dFactor;
	m_adK0Limit[0][1]=m_adK0[0]*dFactor;
	m_adK0Limit[1][0]=m_adK0[1]/dFactor;
	m_adK0Limit[1][1]=m_adK0[1]*dFactor;

	double dlnFactor = log( dFactor );
	m_adK1Limit[0][0]=m_adK1[0]/dlnFactor;
	m_adK1Limit[0][1]=m_adK1[0]*dlnFactor;
	m_adK1Limit[1][0]=m_adK1[1]/dlnFactor;
	m_adK1Limit[1][1]=m_adK1[1]*dlnFactor;
	}

double CModelRate :: RateValueK0( bool bFwd, CStimuli * poStim, double t ) { 
	int i=(bFwd?0:1);
	if( poStim == NULL  || ! m_abPDep[i] ) 
		return m_adK0[i];
	return poStim->StimulusValue( m_acPName[i].Buffer(), t ) * m_adK0[i];
	}

double CModelRate :: RateValueK1( bool bFwd, CStimuli * poStim, double t ) { 
	int i=(bFwd?0:1);
	if( poStim == NULL || ! m_abQDep[i] ) 
		return m_adK1[i];
	return poStim->StimulusValue( m_acQName[i].Buffer(), t ) * m_adK1[i];
	}

double CModelRate :: RateValue( bool bFwd, CStimuli * poStim, double t ) { 
	//-- k0 p e^( k1 q )
	return RateValueK0( bFwd, poStim, t ) * exp( RateValueK1( bFwd, poStim, t ) );
	}
	
	
//-----------------------------------------------------------------------------
CModelConstraint :: CModelConstraint( int iType, int iStates, int * aiStates, double dX, double dY ) { 
	bool bCloseLoop=false;
	if( (iType == CONSTRAINT_LOOPBALANCE  || iType == CONSTRAINT_LOOPIMBALANCE ) 
		&& aiStates!=NULL && aiStates[0]!=aiStates[iStates-1] )  
		bCloseLoop=true;
	m_iStates = bCloseLoop ? iStates+1 : iStates ;
	m_aiStates = new int[m_iStates];
	if( m_aiStates!=NULL && aiStates!=NULL ) {
		for( int i=0; i<iStates; ++i ) 
			m_aiStates[i]=aiStates[i];
		if( bCloseLoop ) 
			m_aiStates[iStates]=aiStates[0];
		}
	m_iType = m_aiStates==NULL ? CONSTRAINT_ERROR : iType;
	m_dXPos = dX;
	m_dYPos = dY;
	}
	
CModelConstraint :: CModelConstraint( const CModelConstraint & refConstraint ) { 
	m_iType = refConstraint.m_iType;
	m_iStates = refConstraint.m_iStates;				
	if( refConstraint.m_aiStates != NULL ) { 
		m_aiStates = new int[m_iStates];
		for( int iState=0; iState<m_iStates; ++iState ) 
			m_aiStates[iState]=refConstraint.m_aiStates[iState];
		}
	else 
		m_aiStates=NULL;
	m_dXPos = refConstraint.m_dXPos;
	m_dYPos = refConstraint.m_dYPos;
	}
	
CModelConstraint :: ~CModelConstraint() { 
	delete [] m_aiStates;
	}

void CModelConstraint :: SetStates( int iStates, int i0, int i1, int i2, int i3 ) {
	if( m_aiStates != NULL ) 
		delete [] m_aiStates;
	m_aiStates = new int[m_iStates=iStates];
	if( m_iStates>0 ) 
		m_aiStates[0]=i0;
	if( m_iStates>1 ) 
		m_aiStates[1]=i1;
	if( m_iStates>2 ) 
		m_aiStates[2]=i2;
	if( m_iStates>3 ) 
		m_aiStates[3]=i3;
	}


void CModelConstraint :: Text( char * pchBuf ) { 
	//----- Text.
	if( m_iType==CONSTRAINT_FIXRATE )
		sprintf( pchBuf, "k0 %d->%d", m_aiStates[0], m_aiStates[1]);
	else if( m_iType==CONSTRAINT_FIXEXP )
		sprintf( pchBuf, "k1 %d->%d", m_aiStates[0], m_aiStates[1]);
	else if( m_iType==CONSTRAINT_SCALERATE )
		sprintf( pchBuf, "k0 %d->%d:%d->%d", m_aiStates[0], m_aiStates[1], m_aiStates[2], m_aiStates[3] );
	else if( m_iType==CONSTRAINT_SCALEEXP )
		sprintf( pchBuf, "k1 %d->%d:%d->%d", m_aiStates[0], m_aiStates[1], m_aiStates[2], m_aiStates[3] );
	else 
		sprintf( pchBuf, "?" );
	}
	
//-----------------------------------------------------------------------------
// Model Merge Constructor 
CModel :: CModel( CModel * pModel1, CModel * pModel2, int iChannels1, int iChannels2 ) { 

	//-- create metamodels
	CMetaModel o1( pModel1, iChannels1, true );
	int iClass1, iClasses1=o1.m_aClasses.Count();
	int iSt1, iStates1 = o1.m_aStates.Count();
	
	int iClass2=0, iClasses2=1;
	int iSt2, iStates2=1;
	CModelClass * pClass2Base = NULL; 
	CMetaModel o2( pModel2==NULL?pModel1:pModel2, iChannels2, true );
	if( pModel2 ) {
		iClasses2 = o2.m_aClasses.Count();
		iStates2 = o2.m_aStates.Count();
		pClass2Base = o2.Class(0);
		}

	//-- merge classes and states
	CModelClass * pClass1, * pClass2;
	CModelState * pState1, * pState2;
	double dAmp, dStd, dProb1, dProb2=1.0;
	TNT::Array2D<short>aiMapClass( iClasses1, iClasses2, -1 ); 
	TNT::Array2D<short>aiMapState( iStates1, iStates2, -1 ); 

	for( iSt1=0; iSt1<iStates1; ++iSt1 ) {
		pState1 = o1.State(iSt1);
		iClass1 = pState1->m_iClass;
		dProb1=pState1->m_dStartProb;
		
		for( iSt2=0; iSt2<iStates2 || iSt2==0; ++iSt2 ) {
			if( pModel2 ) { 
				pState2 = o2.State(iSt2);
				dProb2 = pState2->m_dStartProb;
				iClass2 = pState2->m_iClass;
				}
			
			//--If first state1/state2 combo of this class1/class2 - add merge state  
			if( aiMapClass[iClass1][iClass2]==-1 ) { 
				pClass1 = o1.Class(iClass1);
				if( pModel2 ) { 
					pClass2 = o2.Class(iClass2);
					dAmp = pClass1->m_dAmp + pClass2->m_dAmp - pClass2Base->m_dAmp;
					dStd = sqrt( pClass1->m_dStdDev*pClass1->m_dStdDev + pClass2->m_dStdDev*pClass2->m_dStdDev - pClass2Base->m_dStdDev*pClass2Base->m_dStdDev );
					AddClass( dAmp , dStd );			
					}
				else
					AddClass( pClass1->m_dAmp, pClass1->m_dStdDev );			
				aiMapClass[iClass1][iClass2]=m_aClasses.Count()-1;
				}
				
			//-- Add new state 
			aiMapState[iSt1][iSt2]=m_aStates.Count();
			AddState( aiMapClass[iClass1][iClass2],							// New Class
					  MulDiv(iSt1, 200, iStates1)-100+(100/iStates1),		// X Pos
					  MulDiv(iSt2, 200, iStates2)-100+(100/iStates2),		// Y Pos
					  dProb1 * dProb2 );		// Start prob
			}
		}

	//-- merge rates
	int iPath1, iPaths1=o1.m_aPaths.Count();
	int aiRef[8];
	double dXPos=80.0, dYPos=80.0;
	CModelRate * pRate;
	CModelRate oRate;
	
	for( iPath1=0; iPath1<iPaths1; ++iPath1 ) { 
		for( iSt2=0; iSt2<iStates2; ++iSt2 ) {
			pRate = new CModelRate;
			o1.MakeRate( iPath1, pRate );
			pRate->m_aiStates[0] = aiMapState[o1.MetaPath(iPath1)->m_aiStates[0]][iSt2];	
			pRate->m_aiStates[1] = aiMapState[o1.MetaPath(iPath1)->m_aiStates[1]][iSt2];
			m_aRates.AddItem( pRate );						
			
			if( iSt2==0 ) { 
				aiRef[5]=aiRef[0]=pRate->m_aiStates[0];
				aiRef[4]=aiRef[1]=pRate->m_aiStates[1];
				}
			else { 
				// add constraint [ alternate positioning : -75.0, MulDiv(iSt2*iPaths1+iPath1, 100, iStates2*iPaths1)  ] 
				aiRef[7]=aiRef[2]=pRate->m_aiStates[0];
				aiRef[6]=aiRef[3]=pRate->m_aiStates[1];
				
				if( dYPos<20 ) {
					dXPos=max(20.0, dXPos-CONSTR_XOFFSET);
					dYPos=80.0;
					}
				else 
					dYPos-=CONSTR_YOFFSET*2;
				AddConstraint( CONSTRAINT_SCALERATE, 4, aiRef, dXPos, dYPos );	
				AddConstraint( CONSTRAINT_SCALERATE, 4, aiRef+4, dXPos, dYPos-CONSTR_YOFFSET );	
				if( pRate->m_abQDep[0] ) 
					AddConstraint( CONSTRAINT_SCALEEXP, 4, aiRef, dXPos+1.0, dYPos );	
				if( pRate->m_abQDep[1] ) 
					AddConstraint( CONSTRAINT_SCALEEXP, 4, aiRef+4, dXPos+1.0, dYPos-CONSTR_YOFFSET );	
					
				}
			}
		}

	if( pModel2 ) { 		
		int iPath2, iPaths2=o2.m_aPaths.Count();
		for( iPath2=0; iPath2<iPaths2; ++iPath2 ) { 
			for( iSt1=0; iSt1<iStates1; ++iSt1 ) {
				pRate = new CModelRate;
				o2.MakeRate( iPath2, pRate );
				pRate->m_aiStates[0] = aiMapState[iSt1][o2.MetaPath(iPath2)->m_aiStates[0]];
				pRate->m_aiStates[1] = aiMapState[iSt1][o2.MetaPath(iPath2)->m_aiStates[1]];
				m_aRates.AddItem( pRate );						

				if( iSt1==0 ) { 
					aiRef[5]=aiRef[0]=pRate->m_aiStates[0];
					aiRef[4]=aiRef[1]=pRate->m_aiStates[1];
					}
				else { 
					if( dYPos<20 ) {
						dXPos=max(20.0, dXPos-CONSTR_XOFFSET);
						dYPos=80.0;
						}
					else 
						dYPos-=CONSTR_YOFFSET*2;
					// add constraint 
					aiRef[7]=aiRef[2]=pRate->m_aiStates[0];
					aiRef[6]=aiRef[3]=pRate->m_aiStates[1];
					AddConstraint( CONSTRAINT_SCALERATE, 4, aiRef, dXPos, dYPos );	
					AddConstraint( CONSTRAINT_SCALERATE, 4, aiRef+4, dXPos, dYPos-CONSTR_YOFFSET );	
					if( pRate->m_abQDep[0] ) 
						AddConstraint( CONSTRAINT_SCALEEXP, 4, aiRef, dXPos+1.0, dYPos );	
					if( pRate->m_abQDep[1] ) 
						AddConstraint( CONSTRAINT_SCALEEXP, 4, aiRef+4, dXPos+1.0, dYPos-CONSTR_YOFFSET );	
					}
				}
			}
		}

	//-- Constraints from metamodels
	int iConstraint, iConstraints;
	int iState;

	iConstraints=o1.m_aConstraints.Count();
	CModelConstraint * pCon;
	for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		pCon = new CModelConstraint( *(o1.Constraint(iConstraint)) );
		for( iState=0; iState<pCon->m_iStates; ++iState ) 
			pCon->m_aiStates[iState] = aiMapState[pCon->m_aiStates[iState]][0];
		m_aConstraints.AddItem( pCon );
		}

	if( pModel2 ) { 
		iConstraints=o2.m_aConstraints.Count();
		CModelConstraint * pCon;
		for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
			pCon = new CModelConstraint( *(o2.Constraint(iConstraint)) );
			for( iState=0; iState<pCon->m_iStates; ++iState ) 
				pCon->m_aiStates[iState] = aiMapState[0][pCon->m_aiStates[iState]];
			m_aConstraints.AddItem( pCon );
			}
		}
	/*
	//-- Constraints from original models
	iConstraints=pModel1->m_aConstraints.Count();
	CModelConstraint * pCon;
	for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		pCon = new CModelConstraint( *(pModel1->Constraint(iConstraint)) );
		for( iState=0; iState<pCon->m_iStates; ++iState ) 
			pCon->m_aiStates[iState] = aiMapState[pCon->m_aiStates[iState]][0];
		m_aConstraints.AddItem( pCon );
		}
		
	if( pModel2 ) { 
		iConstraints=pModel2->m_aConstraints.Count();
		for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
			pCon = new CModelConstraint( *(pModel2->Constraint(iConstraint)) );
			for( iState=0; iState<pCon->m_iStates; ++iState ) 
				pCon->m_aiStates[iState] = aiMapState[0][pCon->m_aiStates[iState]];
			m_aConstraints.AddItem( pCon );
			}
		}
	*/
	
	
	}

//-----------------------------------------------------------------------------
//-- default constructor 
CModel :: CModel( ) {
	m_iStatus=0;
	}
	
//-- copy constructor 
CModel :: CModel( const CModel & refModel ) { 
	Copy( refModel );
	}

//-- assignment operator
const CModel & CModel :: operator=( const CModel & refModel ) { 
	Copy( refModel );
	return *this;
	}

//-- copy contents of one model to another 
void CModel :: Copy( const CModel & refModel ) { 
	int i;
	m_iStatus=refModel.m_iStatus;
	
	for( i=0; i<refModel.m_aClasses.Count(); ++i ) 
		m_aClasses.AddItem( new CModelClass( *refModel.Class(i) ) );
		
	for( i=0; i<refModel.m_aStates.Count(); ++i ) 
		m_aStates.AddItem( new CModelState( *refModel.State(i) ) );
		
	for( i=0; i<refModel.m_aRates.Count(); ++i )
		m_aRates.AddItem( new CModelRate( *refModel.Rate(i) ) );
		
	for( i=0; i<refModel.m_aConstraints.Count(); ++i ) 
		m_aConstraints.AddItem( new CModelConstraint( *refModel.Constraint(i) ) );
	}

void CModel :: ClearModel( ) { 
	m_aClasses.DeleteAll();
	m_aStates.DeleteAll();
	m_aRates.DeleteAll();
	m_aConstraints.DeleteAll();
	}
	
void CModel :: NewModel( ) { 
	m_iStatus=0;
	
	CModelClass * pClass;
	for( int i=0; i<10; ++i ) 
		m_aClasses.AddItem( pClass = new CModelClass( 1.0*i, 0.10+0.05*i, 0.0, DefaultClassColor(i) ) );
	
	m_aStates.AddItem( new CModelState( 0, -50.0, 0.0, 1.0 ) );
	m_aStates.AddItem( new CModelState( 1,   0.0, 0.0, 0.0 ) );
	m_aStates.AddItem( new CModelState( 2,  50.0, 0.0, 0.0 ) );
	
	m_aRates.AddItem( new CModelRate( 0, 1 ) );
	m_aRates.AddItem( new CModelRate( 1, 2 ) );
	}
	

bool CModel :: OpenQmf( const char * pchFileName ) { 
	CQMFFile oFile;
		
	bool bErr=false;
	int ai[2];		// buffer for loaded data 

	if( ! oFile.Open( pchFileName ) ) 
		return false;

	//----- scan all states
	//-- go to 'States' group node, then first child
	if( ! oFile.GoRoot() 
		|| ! oFile.GoChild( "States" )
		|| ! oFile.GoChild() ) 
		return false;
	//-- load state and move on to next
	do { 
		CModelState * pState = new CModelState();
		m_aStates.AddItem( pState );
		if( ! oFile.GoChild() ) 
			return false;
		//-- scan child nodes for nodes named { x y class pr } and load info if found
		do { 
			//-- x
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"x")==0 ){
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pState->m_dXPos), 8 );
				pState->m_dXPos=pState->m_dXPos*2.0-100.0;
				}
			//-- Y (becomes z)
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"y")==0 ){
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pState->m_dYPos), 8 );
				pState->m_dYPos=pState->m_dYPos*2.0-100.0;
				}
		
			//-- Class
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"Class")==0 )
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pState->m_iClass), 4 );
			//-- Pr
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"Pr")==0 )
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pState->m_dStartProb), 8 );
			//-- Gr
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"Gr")==0 )
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pState->m_iGroup), 4 );
				
			} while( oFile.GoSibling() );
		if( ! oFile.GoParent() ) 
			return false;
		} while( oFile.m_oCurrent.m_iSiblingPos>=12 && (bErr=oFile.GoSibling()) );
		// if bErr .... 
		
	//----- scan all rates
	//-- go to 'Rates' group node, then first child
	if( ! oFile.GoRoot() 
		|| ! oFile.GoChild( "Rates" )
		|| ! oFile.GoChild() ) 
		return false;
	//-- load Rate and move on to next
	do { 
		CModelRate * pRate = new CModelRate();
		m_aRates.AddItem( pRate );
		if( ! oFile.GoChild() ) 
			return false;
		//-- scan child nodes for nodes named { x y class pr } and load info if found
		do { 
			//-- States
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"States")==0 )
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, pRate->m_aiStates, 4*2 );
			//-- k0
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"k0")==0 )
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, pRate->m_adK0, 8*2 );
			//-- k1
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"k1")==0 )
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, pRate->m_adK1, 8*2 );
			//-- P
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"P")==0 ) {
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, ai, 4*2 );
				pRate->m_abPDep[0]=(ai[0]!=0);
				pRate->m_abPDep[1]=(ai[1]!=0);
				}
			//-- Q
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"Q")==0 ) {
				bErr|=oFile.m_oCurrent.DataBlock( oFile.m_hFile, ai, 4*2 );
				pRate->m_abQDep[0]=(ai[0]!=0);
				pRate->m_abQDep[1]=(ai[1]!=0);
				}
			//-- PNames
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"PNames")==0 ) {
				if( oFile.GoChild() ) { 
					pRate->m_acPName[0].Set( oFile.m_oCurrent.DataString(oFile.m_hFile).Buffer() );
					if( oFile.GoSibling() ) 
						pRate->m_acPName[1].Set( oFile.m_oCurrent.DataString(oFile.m_hFile).Buffer() );
					oFile.GoParent();
					}
				}
			//-- QNames
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"QNames")==0 ) {
				if( oFile.GoChild() ) { 
					pRate->m_acQName[0].Set( oFile.m_oCurrent.DataString(oFile.m_hFile).Buffer() );
					if( oFile.GoSibling() ) 
						pRate->m_acQName[1].Set( oFile.m_oCurrent.DataString(oFile.m_hFile).Buffer() );
					oFile.GoParent();
					}
				}
			//-- PValue, QValue ignored.
			} while( oFile.GoSibling() );
			
		if( ! oFile.GoParent() ) 
			return false;
		} while( oFile.m_oCurrent.m_iSiblingPos>=12 && (bErr=oFile.GoSibling()) );
		// if bErr .... 
	
	//----- Constraints : scan all for types FixRate ScaleRate FixExp ScaleExp 
	//-- go to 'Constraints' group node, then first child
	if( oFile.GoRoot() && oFile.GoChild( "Constraints" ) && oFile.GoChild() ) { 
		//-- load Constraints 
		CModelConstraint * pCons;
		int iConType, iConStates, aiConStates[4];
		bool bMatch;
		do { 
			//-- is current constraint one of the accepted constraint types
			bMatch=false;
			if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"FixRate")==0 ) {
				bMatch=true;
				iConStates=2;
				iConType=CONSTRAINT_FIXRATE;
				}
			else if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"ScaleRate")==0 ) {
				bMatch=true;
				iConStates=4;
				iConType=CONSTRAINT_SCALERATE;
				}
			else if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"FixExp")==0 ) {
				bMatch=true;
				iConStates=2;
				iConType=CONSTRAINT_FIXEXP;
				}
			else if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"ScaleExp")==0 ) {
				bMatch=true;
				iConStates=4;
				iConType=CONSTRAINT_SCALEEXP;
				}
			
			//-- accepted - process it.
			if( bMatch ) {
				// add new constraint.  Caution - expand aiConStates if loops might be 
				oFile.m_oCurrent.DataBlock( oFile.m_hFile, aiConStates, iConStates*sizeof(long) );
				m_aConstraints.AddItem( pCons = new CModelConstraint( iConType, iConStates, aiConStates ) );

				if( oFile.GoChild() ) {
					//-- scan child nodes for nodes named "x" and "y"
					do { 
						if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"x")==0 ){
							oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pCons->m_dXPos), 8 );
							pCons->m_dXPos=pCons->m_dXPos*2.0-100.0;
							}
						if( strcmp(oFile.m_oCurrent.m_cName.Buffer(),"y")==0 ){
							oFile.m_oCurrent.DataBlock( oFile.m_hFile, &(pCons->m_dYPos), 8 );
							pCons->m_dYPos=pCons->m_dYPos*2.0-100.0;
							}
						} while( oFile.GoSibling() );
					oFile.GoParent();
					}
				}
			} while( oFile.m_oCurrent.m_iSiblingPos>=12 && (bErr=oFile.GoSibling()) );
		}
		
	//----- Scan all classes 
	int iAmp, iStd, iCond;
	double * adAmp=NULL, * adStd=NULL, * adCond=NULL;
	bool bSuccess=true;
	
	if( oFile.GoRoot() 
			&& oFile.GoChild( "Amps" )
			&& (adAmp = new double[iAmp=oFile.m_oCurrent.m_iCount]) != NULL 
			&& oFile.m_oCurrent.DataBlock( oFile.m_hFile, adAmp, 8*iAmp ) 

			&& oFile.GoRoot() 
			&& oFile.GoChild( "Stds" )
			&& (adStd = new double[iStd=oFile.m_oCurrent.m_iCount]) != NULL 
			&& oFile.m_oCurrent.DataBlock( oFile.m_hFile, adStd, 8*iStd ) 

			&& oFile.GoRoot() 
			&& oFile.GoChild( "Conds" )
			&& (adCond = new double[iCond=oFile.m_oCurrent.m_iCount]) != NULL 
			&& oFile.m_oCurrent.DataBlock( oFile.m_hFile, adCond, 8*iCond )
			) {
		//-- build class array 
		for( int iClass=0; iClass<min(iAmp,iStd); ++iClass )
			m_aClasses.AddItem( new CModelClass( adAmp[iClass], adStd[iClass], adCond[iClass] ) );

		//-- load colors or set to default
		int iCl=0;
		if( oFile.GoRoot() && oFile.GoChild("Colors") ) {
			COLORREF * adColor = new COLORREF[oFile.m_oCurrent.m_iCount];
			oFile.m_oCurrent.DataBlock( oFile.m_hFile, adColor, 4*oFile.m_oCurrent.m_iCount );
			for( iCl=0; iCl<m_aClasses.Count() && iCl<oFile.m_oCurrent.m_iCount; ++iCl )
				Class(iCl)->m_cr = adColor[iCl];
			delete [] adColor;
			}
		for( /*NULL*/; iCl<m_aClasses.Count(); ++iCl ) 
			Class(iCl)->m_cr = DefaultClassColor(iCl);
		}
	else
		bSuccess=false;
		
	if( adCond!=NULL ) 
		delete [] adCond;
	if( adStd!=NULL ) 
		delete [] adStd;
	if( adAmp!=NULL ) 
		delete [] adAmp;
		
	return bSuccess;
	} 

//-----------------------------------------------------------------------------
bool CModel :: WriteQmf( const char * pchFileName ) { 
	//---------- Create disk file 
	CQMFFile oFile;
	if(  ! oFile.Create( pchFileName, "ModelFile" ) ) 
		return false;
	
	//---------- Write Classes {Amps,Stds,Conds}
	int iClass, iClasses=m_aClasses.Count();
	double * ad = new double[iClasses];
	if( ad == NULL ) 
		return false;

	for( iClass=0; iClass<iClasses; ++iClass ) 
		ad[iClass]=Class(iClass)->m_dAmp;
	oFile.GoRoot();
	oFile.AddNode( "Amps", true, QTP_DOUBLE, 8, iClasses, ad, iClasses*sizeof(double) );

	for( iClass=0; iClass<iClasses; ++iClass ) 
		ad[iClass]=Class(iClass)->m_dStdDev;
	oFile.GoRoot();
	oFile.AddNode( "Stds", true, QTP_DOUBLE, 8, iClasses, ad, iClasses*sizeof(double) );
	
	for( iClass=0; iClass<iClasses; ++iClass ) 
		ad[iClass]=Class(iClass)->m_dConditioning;
	oFile.GoRoot();
	oFile.AddNode( "Conds", true, QTP_DOUBLE, 8, iClasses, ad, iClasses*sizeof(double) );

	delete [] ad;
	
	//-- write colors array - COLORREF as long int's.
	COLORREF * acr = new COLORREF[iClasses];
	for( iClass=0; iClass<iClasses; ++iClass ) 
		acr[iClass]=Class(iClass)->m_cr;
	oFile.GoRoot();
	oFile.AddNode( "Colors", true, QTP_LONG, sizeof(long), iClasses, acr, iClasses*sizeof(long) );
	delete [] acr;

	//---------- Write States 	
	double dX,dY;
	oFile.GoRoot();
	oFile.AddNode( "States", true, QTP_EMPTY, 0, 0 );
	
	int iState, iStates=m_aStates.Count();
	for( iState=0; iState<iStates; ++iState ) {
		CModelState * pState = State(iState);
		dX=(pState->m_dXPos+100.0)/2.0;
		dY=(pState->m_dYPos+100.0)/2.0;
		
		oFile.AddNode( "State", true, QTP_EMPTY, 0, 0 );
		oFile.AddNode( "x", true, QTP_DOUBLE, 8, 1, &dX, 8 );
		oFile.AddNode( "y", false, QTP_DOUBLE, 8, 1, &dY, 8 );
		oFile.AddNode( "Class", false, QTP_LONG, 4, 1, &pState->m_iClass, 4 );
		oFile.AddNode( "Pr", false, QTP_DOUBLE, 8, 1, &pState->m_dStartProb, 8 );
		oFile.AddNode( "Gr", false, QTP_LONG, 4, 1, &pState->m_iGroup, 4 );
		oFile.GoParent();
		oFile.GoParent();
		}
		
	//---------- Write Rates 
	int aiTemp[2];
	oFile.GoRoot();
	oFile.AddNode( "Rates", true, QTP_EMPTY, 0, 0 );
	
	int iRate, iRates=m_aRates.Count();
	for( iRate=0; iRate<iRates; ++iRate ) { 
		CModelRate * pRate = Rate( iRate );
		
		oFile.AddNode( "Rate", true, QTP_EMPTY, 0, 0 );
		oFile.AddNode( "States", true, QTP_LONG, 8, 2, pRate->m_aiStates, 2*4 );
		oFile.AddNode( "k0", false, QTP_DOUBLE, 8, 2, pRate->m_adK0, 2*8 );
		oFile.AddNode( "k1", false, QTP_DOUBLE, 8, 2, pRate->m_adK1, 2*8 );
		aiTemp[0]=pRate->m_abPDep[0]?1:0;
		aiTemp[1]=pRate->m_abPDep[1]?1:0;
		oFile.AddNode( "P", false, QTP_LONG, 8, 2, aiTemp, 2*4 );
		aiTemp[0]=pRate->m_abQDep[0]?1:0;
		aiTemp[1]=pRate->m_abQDep[1]?1:0;
		oFile.AddNode( "Q", false, QTP_LONG, 8, 2, aiTemp, 2*4 );

		oFile.AddNode( "PNames", false, QTP_EMPTY, 0, 0 );
		oFile.AddStringNode( "PName", true, &(pRate->m_acPName[0]), true);
		oFile.AddStringNode( "PName", false, &(pRate->m_acPName[1]), true);
		oFile.GoParent();

		oFile.AddNode( "QNames", false, QTP_EMPTY, 0, 0 );
		oFile.AddStringNode( "QName", true, &(pRate->m_acQName[0]), true);
		oFile.AddStringNode( "QName", false, &(pRate->m_acQName[1]), true);
		oFile.GoParent();

		// dk0 dk1 PValue QValue
		oFile.GoParent();
		oFile.GoParent();
		}	

	//---------- Write Constraints 	
	oFile.GoRoot();
	oFile.AddNode( "Constraints", true, QTP_EMPTY, 0, 0 );
	
	int iConStates, iCon, iCons=m_aConstraints.Count();
	char * pchCon;
	bool bFirstCon=true;
	for( iCon=0; iCon<iCons; ++iCon ) {
		CModelConstraint * pCon = Constraint(iCon);
		pchCon = NULL;
		if( pCon->m_iType==CONSTRAINT_FIXRATE ) {
			pchCon= "FixRate";
			iConStates = 2;
			}
		else if( pCon->m_iType==CONSTRAINT_FIXEXP ) {
			pchCon= "FixExp";
			iConStates = 2;
			}
		else if( pCon->m_iType==CONSTRAINT_SCALERATE ) {
			pchCon= "ScaleRate";
			iConStates = 4;
			}
		else if( pCon->m_iType==CONSTRAINT_SCALEEXP ) {
			pchCon= "ScaleExp";
			iConStates = 4;
			}
		if( pchCon != NULL ) { 
			dX=(pCon->m_dXPos+100.0)/2.0;
			dY=(pCon->m_dYPos+100.0)/2.0;
			oFile.AddNode( pchCon, bFirstCon, QTP_LONG, 4, iConStates, pCon->m_aiStates, 4*iConStates );
			oFile.AddNode( "x", true, QTP_DOUBLE, 8, 1, &dX, 8 );
			oFile.AddNode( "y", false, QTP_DOUBLE, 8, 1, &dY, 8 );
			oFile.GoParent();
			bFirstCon=false;
			}
		}

	oFile.Close();
	return true;
	} 

//-----------------------------------------------------------------------------
CModelRate * CModel :: Rate( int i ) const {
	return (CModelRate *)m_aRates.GetAt(i);
	}

CModelState * CModel :: State( int i ) const { 
	return (CModelState *)m_aStates.GetAt(i);
	}		

CModelClass * CModel :: Class( int i ) const { 
	return (CModelClass *)m_aClasses.GetAt(i);
	}

CModelConstraint * CModel :: Constraint( int i ) const {
	return (CModelConstraint *)m_aConstraints.GetAt(i);
	}

bool CModel :: HasPreExpDepend() {
	for( int iRate=0; iRate<m_aRates.Count(); ++iRate )
		if( ((CModelRate *)m_aRates[iRate])->m_abPDep[0] || ((CModelRate *)m_aRates[iRate])->m_abPDep[1] ) 
			return true;
	return false;
	}

bool CModel :: HasExpDepend() { 
	for( int iRate=0; iRate<m_aRates.Count(); ++iRate )
		if( ((CModelRate *)m_aRates[iRate])->m_abQDep[0] || ((CModelRate *)m_aRates[iRate])->m_abQDep[1] ) 
			return true;
	return false;
	}
	
void CModel :: GetStimuliiNames( CStringArray & aNames ) {
	for( int iRate=0; iRate<m_aRates.Count(); ++iRate ) { 
		CModelRate * pRate = (CModelRate *)m_aRates[iRate];
		if( pRate->m_abPDep[0] && aNames.Find( pRate->m_acPName[0].ConstBuffer())==-1 ) 
			aNames.AddString( pRate->m_acPName[0].ConstBuffer() );
		if( pRate->m_abPDep[1] && aNames.Find( pRate->m_acPName[1].ConstBuffer())==-1 ) 
			aNames.AddString( pRate->m_acPName[1].ConstBuffer() );
		if( pRate->m_abQDep[0] && aNames.Find( pRate->m_acQName[0].ConstBuffer())==-1 ) 
			aNames.AddString( pRate->m_acQName[0].ConstBuffer() );
		if( pRate->m_abQDep[1] && aNames.Find( pRate->m_acQName[1].ConstBuffer())==-1 ) 
			aNames.AddString( pRate->m_acQName[1].ConstBuffer() );
		}
	}
			
bool CModel :: AddState( int iClass, double dXPos, double dZPos, double dProb ) { 
	CModelState * pState = new CModelState;
	if( pState==NULL ) 
		return false;
	pState->m_iClass=iClass;
	pState->m_dStartProb=dProb;
	pState->m_dXPos=dXPos;
	pState->m_dYPos=dZPos;
	m_aStates.AddItem( pState );
	return true;
	}

bool CModel :: AddConstraint( int iType, int iStates, int * aiStates, double dX, double dY ) { 
	CModelConstraint * pConstraint = new CModelConstraint( iType, iStates, aiStates, dX, dY );
	if( pConstraint==NULL ) 
		return false;
	m_aConstraints.AddItem( pConstraint );
	return true;
	}

bool CModel :: DeleteConstraint( int iConstraint ) { 
	return m_aConstraints.DeleteItem( iConstraint );
	} 

bool CModel :: DeleteState( int iState ) { 
	bool bRet=true;
	CModelRate * pRate;
	for( int iRate=m_aRates.Count()-1; iRate>=0; --iRate ) {
		pRate = (CModelRate *)m_aRates[iRate];
	
		if( pRate->m_aiStates[0]==iState || pRate->m_aiStates[1]==iState ) {
			if( ! m_aRates.DeleteItem( iRate ) )
				bRet=false;
			}
		else { 
			if( pRate->m_aiStates[0]>iState ) 
				pRate->m_aiStates[0]--;
			if( pRate->m_aiStates[1]>iState ) 
				pRate->m_aiStates[1]--;
			}
		}	
				
	return bRet && m_aStates.DeleteItem( iState );	
	}
	
bool CModel :: AddRate( int iState0, int iState1 ) { 
	CModelRate * pRate = new CModelRate;
	if( pRate==NULL ) 
		return false;
	pRate->m_aiStates[0]=iState0;
	pRate->m_aiStates[1]=iState1;
	pRate->m_adK0[0]=25.0;				//TODO - ability to change default rates 
	pRate->m_adK0[1]=25.0;				
	pRate->m_adK1[0]=0.0;
	pRate->m_adK1[1]=0.0;	
	pRate->m_abPDep[0]=false;
	pRate->m_abPDep[1]=false;
	pRate->m_abQDep[0]=false;
	pRate->m_abQDep[1]=false;
	pRate->m_acPName[0].Set("Ligand");
	pRate->m_acPName[1].Set("Ligand");
	pRate->m_acQName[0].Set("Voltage");
	pRate->m_acQName[1].Set("Voltage");
	m_aRates.AddItem( pRate );
	return true;
	}
				
bool CModel :: DeleteRate( int iRate ) { 
	return m_aRates.DeleteItem( iRate );
	} 

CModelClass * CModel :: AddClass( double dAmp, double dStd, double dCond ) { 
	CModelClass * pClass = new CModelClass( dAmp, dStd, dCond );
	pClass->m_cr = DefaultClassColor(m_aClasses.Count());
	m_aClasses.AddItem( pClass );
	return pClass;
	}

void CModel :: AddClass() { 
	CModelClass * pClass, * pClass1, * pClass2;
	int iClasses =  m_aClasses.Count();
	if( iClasses>=MAXCLASSES )	// should not be possible
		return;
			
	if( iClasses>=2 ) {			// new class = last class + difference between last class and prior class.
		pClass1 = Class(iClasses-1);
		pClass2 = Class(iClasses-2);
		pClass = new CModelClass( 2.0*pClass1->m_dAmp-pClass2->m_dAmp, 
								  2.0*pClass1->m_dStdDev-pClass2->m_dStdDev, 
								  2.0*pClass1->m_dConditioning-pClass2->m_dConditioning );
		}
	else if( iClasses>=1 ) {	// New class based on existing class
		pClass1 = Class(iClasses-1);
		pClass = new CModelClass( pClass1->m_dAmp*2.0, 
								  pClass1->m_dStdDev*1.5, 
								  pClass1->m_dConditioning );
		}			
	else 
		pClass = new CModelClass( 1.0, 0.10, 1.0 );
	pClass->m_cr = DefaultClassColor(iClasses);
	m_aClasses.AddItem( pClass );
	}
	
void CModel :: DeleteClass( int iClass ) { 
	int iClasses = m_aClasses.Count();
	if( iClass<0 || iClass>=iClasses || iClasses<3 ) 
		return;
	m_aClasses.DeleteItem( iClass );
	--iClasses;
	int iStates=m_aStates.Count();
	for( int iState=0; iState<iStates; ++iState ) 
		if( State(iState)->m_iClass>=iClasses ) 
			State(iState)->m_iClass=iClasses-1;
	}

int CModel :: UsedClassCount( Array1D<int> * paiClasses ) {
	int iState, iClass, iClasses=m_aClasses.Count();
	bool * abUsed = new bool[iClasses];
	for( iClass=0; iClass<iClasses; ++iClass )
		abUsed[iClass]=false;
	for( iState=0; iState<m_aStates.Count(); ++iState ) {
		iClass=State(iState)->m_iClass;
		if( iClass>=0 && iClass<iClasses ) 
			abUsed[ iClass ]=true;
		}
		
	int iUsed=0;
	for( iClass=0; iClass<iClasses; ++iClass )
		if( abUsed[iClass] )
			++iUsed;
	
	if( paiClasses != NULL ) {
		*paiClasses = Array1D<int>(iUsed);
		iUsed=0;
		for( iClass=0; iClass<iClasses; ++iClass ) {
			if( abUsed[iClass] ) {
				(*paiClasses)[iUsed++]=iClass;
				}
			}
		}
	
	delete [] abUsed;
	return iUsed;
	}

bool CModel :: UsesClass( int iClass ) {
	for( int i=0; i<m_aStates.Count(); ++i ) 
		if( State(i)->m_iClass==iClass )
			return true;
	return false;
	}

bool CModel :: SetStatus( int iStatus ) { 
	//TODO - protect access to status
	m_iStatus=iStatus;
	return true;
	}

int CModel :: GetStatus() { 
	//TODO - protect access to status
	return m_iStatus;
	}
	
double CModel :: StateAmp( int iState ) { 
	return Class(State(iState)->m_iClass)->m_dAmp;
	}
	
double CModel :: StateStd( int iState ) { 
	return Class(State(iState)->m_iClass)->m_dStdDev;
	}
	
CModelRate * CModel :: FindRate( int iState0, int iState1, bool * pbBwd ) { 
	int iRate = FindRateIndex( iState0, iState1, pbBwd );
	if( iRate==-1 ) 
		return NULL;
	return Rate(iRate);
	}
	
int CModel :: FindRateIndex( int iState0, int iState1, bool * pbBwd ) { 
	CModelRate * pRate;
	if( pbBwd != NULL ) 
		*pbBwd=false;
	int iRates = m_aRates.Count();
	for( int iRate=0; iRate<iRates; ++iRate ) { 
		pRate = Rate( iRate );
		if( pRate->m_aiStates[0]==iState0 && pRate->m_aiStates[1]== iState1 ) 
			return iRate;
		if( pRate->m_aiStates[1]==iState0 && pRate->m_aiStates[0]== iState1 ) {
			if( pbBwd != NULL ) 
				*pbBwd=true;
			return iRate;
			}
		}
	return -1;
	}
	
void CModel :: BuildConstraintsMtxVct( Array2D<double> & Mtx, Array1D<double> & Vct, int * piConstraintRows ) { 
	/* Build the constraints matrix and vector.
		Note : each rate in model corresponds to 4 columns in the Mtx, 
	  for k0[0], k1[0], k0[1], k1[1].  k0's are ln()'d
	  This takes two passes through the constraints list, one to determine 
	  the number of rows the matrix will eventually contain, the second to 
	  build the matrix and vector.
	*/
	CModelConstraint * pCon;
	CModelRate * pRate0, * pRate1;
	int iRate0, iRate1;
	bool bBkwd0, bBkwd1;
	
	int iRate, iState, iStatus, iRow, iRows=0;
	int iRates = m_aRates.Count();
	int iConstraints = m_aConstraints.Count();
	
	//-- Determine the number of specified constraints 
	for( int iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		pCon = Constraint(iConstraint);
		switch( pCon->m_iType ) { 
			case CONSTRAINT_FIXRATE :
			case CONSTRAINT_FIXEXP :
				if( -1!=FindRateIndex( pCon->m_aiStates[0], pCon->m_aiStates[1] ) ) 
					++iRows;
				break;
			case CONSTRAINT_SCALERATE :
				if( -1!=FindRateIndex( pCon->m_aiStates[0], pCon->m_aiStates[1] )
					&& -1!=FindRateIndex( pCon->m_aiStates[2], pCon->m_aiStates[3] ) ) 
					++iRows;
				break;
			case CONSTRAINT_SCALEEXP :
				pRate0 = FindRate( pCon->m_aiStates[0], pCon->m_aiStates[1], &bBkwd0 );
				pRate1 = FindRate( pCon->m_aiStates[2], pCon->m_aiStates[3], &bBkwd1 );
				if( pRate0 && pRate1 && fabs(pRate0->m_adK1[bBkwd0?1:0])>1.0e-6 
						&& fabs(pRate1->m_adK1[bBkwd1?1:0])>1.0e-6 ) 
					++iRows;
				break;
			case CONSTRAINT_LOOPBALANCE :
			case CONSTRAINT_LOOPIMBALANCE :
				iStatus = ConstraintStatus( iConstraint );
				if( iStatus == 0 )
					iRows+=2;
				else if( iStatus == 1 )
					++iRows;
				break;
			}
		}

	//-- count the implicit fixexp constraints for 0.0 exponentials 
	for( iRate=0; iRate<iRates; ++iRate ) {
		pRate0 = Rate( iRate );
		if( fabs(pRate0->m_adK1[0]) < 1.0e-6 ) 
			++iRows;
		if( fabs(pRate0->m_adK1[1]) < 1.0e-6 ) 
			++iRows;
		}
			
	//-- allocate space for the constraints 
	*piConstraintRows = iRows;
	Mtx = Array2D<double>( iRows, 4*iRates, 0.0 );
	Vct = Array1D<double>( iRows, 0.0 );
	
	//-- Fill in the non-zero elements 
	iRow=0;
	for( int iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		pCon = Constraint(iConstraint);
		
		iRate0 = FindRateIndex( pCon->m_aiStates[0], pCon->m_aiStates[1], &bBkwd0 );
		pRate0 = ( iRate0==-1 ? NULL : Rate(iRate0) );

		switch( pCon->m_iType ) { 
			case CONSTRAINT_FIXRATE :
				if( pRate0 ) { 
					Mtx[iRow][iRate0*4+(bBkwd0?2:0)]=1.0;
					Vct[iRow]=log(pRate0->m_adK0[bBkwd0?1:0]);
					//TODO - check for log(0)
					iRow++;
					}
				break;
				
			case CONSTRAINT_FIXEXP :
				if( pRate0 ) { 
					Mtx[iRow][iRate0*4+(bBkwd0?3:1)]=1.0;
					Vct[iRow]=pRate0->m_adK1[bBkwd0?1:0];
					iRow++;
					}
				break;
				
			case CONSTRAINT_SCALERATE :
				iRate1 = FindRateIndex( pCon->m_aiStates[2], pCon->m_aiStates[3], &bBkwd1 );
				pRate1 = ( iRate1==-1 ? NULL : Rate(iRate1) );
				if( pRate0 && pRate1 ) { 
					Mtx[iRow][iRate0*4+(bBkwd0?2:0)]=-1.0;
					Mtx[iRow][iRate1*4+(bBkwd1?2:0)]=1.0;
					Vct[iRow]=log(pRate1->m_adK0[bBkwd1?1:0])-log(pRate0->m_adK0[bBkwd0?1:0]);
					//TODO - check for log(0)
					iRow++;
					}
				break;
			
			case CONSTRAINT_SCALEEXP :
				iRate1 = FindRateIndex( pCon->m_aiStates[2], pCon->m_aiStates[3], &bBkwd1 );
				pRate1 = ( iRate1==-1 ? NULL : Rate(iRate1) );
				if( pRate0 && pRate1 && fabs(pRate0->m_adK1[bBkwd0?1:0])>1.0e-6 
						&& fabs(pRate1->m_adK1[bBkwd1?1:0])>1.0e-6 ) {
					Mtx[iRow][iRate0*4+(bBkwd0?3:1)]=1.0;
					Mtx[iRow][iRate1*4+(bBkwd1?3:1)]
						= -1.0 * pRate0->m_adK1[bBkwd0?1:0] / pRate0->m_adK1[bBkwd1?1:0];
					Vct[iRow]=0.0;
					iRow++;
					}
				break;
			
			case CONSTRAINT_LOOPBALANCE :		// same as loop imbalance except leave vct values 0.0
			case CONSTRAINT_LOOPIMBALANCE :
				iStatus = ConstraintStatus( iConstraint );
				if( iStatus>=0 ) {
					for( iState=0; iState<pCon->m_iStates-1; ++iState ) {
						pRate0 = FindRate( pCon->m_aiStates[iState], pCon->m_aiStates[iState+1], &bBkwd0 );
						Mtx[iRow][iRate0*4+(bBkwd0?2:0)]=1.0;
						Mtx[iRow][iRate0*4+(bBkwd0?0:2)]=-1.0;
						if( pCon->m_iType == CONSTRAINT_LOOPIMBALANCE ) 
							Vct[iRow]+=log(pRate0->m_adK0[bBkwd0?1:0]) - log(pRate0->m_adK0[bBkwd0?0:1]);
						if( iStatus==1 ) { 
							// add a 2nd row for exponentials on loops.
							Mtx[iRow+1][iRate0*4+(bBkwd0?3:1)]=1.0;
							Mtx[iRow+1][iRate0*4+(bBkwd0?1:3)]=-1.0;
							if( pCon->m_iType == CONSTRAINT_LOOPIMBALANCE ) 
								Vct[iRow+1]+=pRate0->m_adK1[bBkwd0?1:0] - pRate0->m_adK1[bBkwd0?0:1];
							}
						}
					iRow += (iStatus+1);
					}
				break;
			}
		}

	//-- Add in the implicit fixexp constraints for 0.0 exponentials 
	for( iRate=0; iRate<iRates; ++iRate ) {
		pRate0 = Rate( iRate );
		if( fabs(pRate0->m_adK1[0]) < 1.0e-6 ) {
			Mtx[iRow][iRate*4+1]=1.0;
			++iRow;
			}
		if( fabs(pRate0->m_adK1[1]) < 1.0e-6 ) {
			Mtx[iRow][iRate*4+3]=1.0;
			++iRow;
			}
		}
		
	} 

void CModel :: SetRateLimits( double dLimitFactor ) { 
	int iRate, iRates=m_aRates.Count();
	for( iRate=0; iRate<iRates; ++iRate ) 
		((CModelRate *)m_aRates[iRate])->SetLimits( dLimitFactor );
	}

int CModel :: ConstrainedRate( int iState0, int iState1 ) { 
	//0=no 1=fwd:fixed 2=fwd:scaled -1=bwd:fixed -2=bwd:scaled
	CModelConstraint * pCon;
	int iConstraints=m_aConstraints.Count();
	for( int iConstraint=0; iConstraint<iConstraints; ++iConstraint ) {
		pCon = Constraint(iConstraint);
		if( pCon->m_aiStates[0]==iState0 && pCon->m_aiStates[1]==iState1 ) {
			if( pCon->m_iType==CONSTRAINT_FIXRATE )
				return 1;
			if( pCon->m_iType==CONSTRAINT_SCALERATE )
				return 2;
			}
		if( pCon->m_aiStates[0]==iState1 && pCon->m_aiStates[1]==iState0 ) {
			if( pCon->m_iType==CONSTRAINT_FIXRATE )
				return -1;
			if( pCon->m_iType==CONSTRAINT_SCALERATE )
				return -2;
			}
		}
	return 0;
	}

/*
bool CModel :: ConstrainedFixRate( int iState0, int iState1 ) { 
	CModelConstraint * pCon;
	int iConstraints=m_aConstraints.Count();
	for( int iConstraint=0; iConstraint<iConstraints; ++iConstraint ) {
		pCon = Constraint(iConstraint);
		if( pCon->m_iType==CONSTRAINT_FIXRATE 
			&& pCon->m_aiStates[0]==iState0 
			&& pCon->m_aiStates[1]==iState1 )
			return true;
		}
	return false;
	}

bool CModel :: ConstrainedScaleRate( int iState0, int iState1 ) { 
	CModelConstraint * pCon;
	int iConstraints=m_aConstraints.Count();
	for( int iConstraint=0; iConstraint<iConstraints; ++iConstraint ) {
		pCon = Constraint(iConstraint);
		if( pCon->m_iType==CONSTRAINT_SCALERATE 
			&& ( ( pCon->m_aiStates[0]==iState0 && pCon->m_aiStates[1]==iState1 ) 
				|| ( pCon->m_aiStates[2]==iState0 && pCon->m_aiStates[3]==iState1 ) ) )
			return true;
		}
	return false;
	}
*/

bool CModel :: ConstrainedLoopRate( bool bImbal, int iState0, int iState1 ) { 
	CModelConstraint * pCon;
	int iState, iConstraints=m_aConstraints.Count();
	for( int iConstraint=0; iConstraint<iConstraints; ++iConstraint ) {
		pCon = Constraint(iConstraint);
		if( pCon->m_iType==(bImbal?CONSTRAINT_LOOPIMBALANCE:CONSTRAINT_LOOPBALANCE) )
			for( iState=0; iState<pCon->m_iStates-1; ++iState ) 
				if( ( pCon->m_aiStates[iState]==iState0 && pCon->m_aiStates[iState+1]==iState1 ) 
					|| ( pCon->m_aiStates[iState+1]==iState0 && pCon->m_aiStates[iState]==iState1 ) )
					return true;
		}
	return false;
	}

int CModel :: ConstraintStatus( int iConstraint ) { 
	CModelConstraint * pCon = Constraint(iConstraint);
	if( pCon->m_iType!=CONSTRAINT_LOOPBALANCE && pCon->m_iType!=CONSTRAINT_LOOPIMBALANCE )
		return -2;

	CModelRate * pRate;
	bool bExp=false;
	for( int i=0; i<pCon->m_iStates-1; ++pCon ) {
		pRate = FindRate( pCon->m_aiStates[i], pCon->m_aiStates[i+1] );
		if( pRate == NULL ) 
			return -1;
		if( fabs(pRate->m_adK1[0])>1.0e-6 || fabs(pRate->m_adK1[1])>1.0e-6 ) 
			bExp=true;
		}
	return bExp?1:0;
	}

//-- Fill in provided arrays with amps and stds for all used classes.
int CModel :: GetAmpStdArray( int iSize, int * aiClass, double * adAmp, double *adStd, bool bSort ) { 
	int iClass, iFilled=0, iClasses=m_aClasses.Count();
	for( iClass=0; iClass<iClasses && iFilled<iSize; ++iClass ) 
		if( UsesClass( iClass ) ) {
			CModelClass * pClass=Class(iClass);
			aiClass[iFilled]=iClass;
			adAmp[iFilled]=pClass->m_dAmp;
			adStd[iFilled]=pClass->m_dStdDev;
			++iFilled;
			}
		
	//-- sort amp and std by amps ascending
	if( bSort ) { 
		iClass=0;
		while( iClass < iFilled-1 ) {
			if( adAmp[iClass]>adAmp[iClass+1] ) {
				Swap( &(aiClass[iClass]), &(aiClass[iClass+1]) );
				Swap( &(adAmp[iClass]), &(adAmp[iClass+1]) );
				Swap( &(adStd[iClass]), &(adStd[iClass+1]) );
				if( iClass>0 ) 
					--iClass;
				}
			else 
				++iClass;
			}
		}
	return iFilled;
	}

double CModel :: NormalizeStartProbabilities(double * ad) { 
	int i, iStates=m_aStates.Count();
	double dSum=0.0, dRevised=0.0;
	
	//-- Note : two methods could be combined using an array of pointers, but less efficient for a small code saving.
	if( ad!=NULL ) {
		for( i=0; i<iStates; ++i ) 
			dSum += (ad[i]=(((CModelState *)m_aStates[i])->m_dStartProb));
		if( fabs( dSum ) < 1.0e-5 )				// if sum(pr) is 0.0  
			for( i=0; i<iStates; ++i ) 
				dRevised += ( ad[i] = 1.0 / double(iStates));
		else if( fabs( dSum - 1.0 ) < 1.0e-5 )		// if sum(pr) is not 1.0  
			for( i=0; i<m_aStates.Count(); ++i ) 
				dRevised += ( ad[i] /= dSum );
		else 
			dRevised=dSum;
		}
	else {
		for( i=0; i<iStates; ++i ) 
			dSum += ((CModelState *)m_aStates[i])->m_dStartProb;
		if( fabs( dSum ) < 1.0e-5 )				// if sum(pr) is 0.0  
			for( i=0; i<iStates; ++i ) 
				dRevised += ( ((CModelState *)m_aStates[i])->m_dStartProb = 1.0 / double(iStates));
		else if( fabs( dSum - 1.0 ) < 1.0e-5 )		// if sum(pr) is not 1.0  
			for( i=0; i<m_aStates.Count(); ++i ) 
				dRevised += ( ((CModelState *)m_aStates[i])->m_dStartProb /= dSum );
		else 
			dRevised=dSum;
		}
	
	return dRevised;
	}

//-----------------------------------------------------------------------------
// Mark constrained rates and set m_dValue[] to scale factor or loop imbalance
bool CModel :: SetConstraintStatus() { 
	bool bError=false;

	//-- set all status to false	
	int iRate, iRates=m_aRates.Count();
	CModelRate *pRate;
	for( iRate=0; iRate<iRates; ++iRate ) {
		pRate = Rate(iRate);
		pRate->m_abConstrainedK0[0]=false;
		pRate->m_abConstrainedK0[1]=false;
		pRate->m_abConstrainedK1[0]=(pRate->m_adK1[0]==0.0);
		pRate->m_abConstrainedK1[1]=(pRate->m_adK1[1]==0.0);
		}

	//--
	int iState; 
	int iRate0, iRate1;
	int iConstraint, iConstraints=m_aConstraints.Count();
	bool bBkwd0, bBkwd1, bK1;
	bool *pbStatus0,*pbStatus1;	
	CModelRate *pRate0, *pRate1;
	CModelConstraint * pCon;
	double dRate1, dRate0;

	//-- all non-loop constraints		
	for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		//-- get constraint
		pCon = Constraint(iConstraint);
		pCon->m_apAdjust[0]=pCon->m_apAdjust[1]=NULL;
		pCon->m_bError=false;
		
		//-- get {iRate0, bBkwd0, pRate0} pointer to rate for first 2 states 
		iRate0 = FindRateIndex( pCon->m_aiStates[0], pCon->m_aiStates[1], &bBkwd0 );
		pRate0 = Rate(iRate0);

		//-- set constraint flags 
		switch( pCon->m_iType ) { 
			case CONSTRAINT_FIXRATE :
			case CONSTRAINT_FIXEXP :
				pbStatus0 = ( pCon->m_iType==CONSTRAINT_FIXEXP ? pRate0->m_abConstrainedK1 : pRate0->m_abConstrainedK0 ) + (bBkwd0?1:0); 
				bError |= (pCon->m_bError=*pbStatus0);		// error if already constrained
				*pbStatus0=true;
				break;
				
			case CONSTRAINT_SCALERATE :
			case CONSTRAINT_SCALEEXP :
				iRate1 = FindRateIndex( pCon->m_aiStates[2], pCon->m_aiStates[3], &bBkwd1 );
				pRate1 = Rate(iRate1);
				if( pRate0==NULL || pRate1==NULL )
					bError=pCon->m_bError=true;
				else { 
					bK1 = pCon->m_iType==CONSTRAINT_SCALEEXP;
					pbStatus0 = ( bK1 ? pRate0->m_abConstrainedK1 : pRate0->m_abConstrainedK0 ) + (bBkwd0?1:0); 
					pbStatus1 = ( bK1 ? pRate1->m_abConstrainedK1 : pRate1->m_abConstrainedK0 ) + (bBkwd1?1:0); 
					dRate0 = ( bK1 ? pRate0->m_adK1 : pRate0->m_adK0 )[bBkwd0?1:0]; 
					dRate1 = ( bK1 ? pRate1->m_adK1 : pRate1->m_adK0 )[bBkwd1?1:0]; 
					bError |= (pCon->m_bError=*pbStatus1);
					*pbStatus1 = true;
					pCon->m_adValue[bK1?1:0] = dRate0==0.0 ? 1.0 : dRate1/dRate0;
					
					pCon->m_apAdjust[0]=pRate0;
					pCon->m_apAdjust[1]=pRate1;
					pCon->m_abAdjustBkwd[0]=bBkwd0;
					pCon->m_abAdjustBkwd[1]=bBkwd1;
					}
				break;
			}
		}

	//-- all loop constraints
	// TODO - depth search for Unconstained rate within loops which do not conflict.
	for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		//-- get constraint
		pCon = Constraint(iConstraint);
		bK1=false;
		
		if( pCon->m_iType==CONSTRAINT_LOOPBALANCE || pCon->m_iType==CONSTRAINT_LOOPIMBALANCE ) { 
			
			//-- find a rate to impose loop balance for k0 and k1
			pCon->m_adValue[0]=pCon->m_adValue[1]=0.0;
			for( iState=0; iState<pCon->m_iStates-1; ++iState ) {
				pRate0 = FindRate( pCon->m_aiStates[iState], pCon->m_aiStates[iState+1], &bBkwd0 );
				bK1 |= ( pRate0->m_adK1[bBkwd0?1:0]!=0.0 );
				if( pCon->m_iType==CONSTRAINT_LOOPIMBALANCE ) { 
					pCon->m_adValue[0] *= pRate0->m_adK0[bBkwd0?1:0] / pRate0->m_adK0[bBkwd0?0:1];
					pCon->m_adValue[1] += pRate0->m_adK1[bBkwd0?1:0] - pRate0->m_adK1[bBkwd0?0:1];
					}
				//-- K1 ptr
				if( pCon->m_apAdjust[1]==NULL && ! pRate0->m_abConstrainedK1[bBkwd0?1:0] ) {
					pCon->m_apAdjust[1] = pRate0;
					pCon->m_abAdjustBkwd[1] = bBkwd0;
					}
				if( pCon->m_apAdjust[1]==NULL && ! pRate0->m_abConstrainedK1[bBkwd0?0:1] ) {
					pCon->m_apAdjust[1] = pRate0;
					pCon->m_abAdjustBkwd[1] = !bBkwd0;
					pCon->m_adValue[1] *= -1.0;
					}
				//-- K0 ptr
				if( pCon->m_apAdjust[0]==NULL && ! pRate0->m_abConstrainedK0[bBkwd0?1:0] ) {
					pCon->m_apAdjust[0] = pRate0;
					pCon->m_abAdjustBkwd[0] = bBkwd0;
					}
				if( pCon->m_apAdjust[0]==NULL && ! pRate0->m_abConstrainedK0[bBkwd0?0:0] && pCon->m_adValue[0]!=0.0 ) {
					pCon->m_apAdjust[0] = pRate0;
					pCon->m_abAdjustBkwd[0] = !bBkwd0;
					pCon->m_adValue[0] = 1.0/pCon->m_adValue[0];
					}
				}
					
			//-- if no rate was found 
			if( ( bK1 && pCon->m_apAdjust[1]==NULL ) || pCon->m_apAdjust[0]==NULL ) {
				bError=pCon->m_bError=true;
				continue;
				}
			
			//-- 
			if( pCon->m_iType==CONSTRAINT_LOOPBALANCE ) { 
				pCon->m_adValue[0] = 1.0;
				pCon->m_adValue[1] = 0.0;
				}
			
			//-- if k1 are not all 0.0, and k0 can use same rate as k1, then it will 
			if( bK1 && ! pCon->m_apAdjust[1]->m_abConstrainedK0[pCon->m_abAdjustBkwd[0]?1:0] ) {
				pCon->m_apAdjust[0]=pCon->m_apAdjust[1];
				pCon->m_abAdjustBkwd[0]=pCon->m_abAdjustBkwd[1];
				}
				
			//-- mark the selected rates 
			pCon->m_apAdjust[0]->m_abConstrainedK0[pCon->m_abAdjustBkwd[0]?1:0]=true;
			if( bK1 ) 
				pCon->m_apAdjust[1]->m_abConstrainedK1[pCon->m_abAdjustBkwd[1]?1:0]=true;
			}
		}

	return bError;
	}
		
//-----------------------------------------------------------------------------
// Mark constrained rates and set m_dValue[] to scale factor or loop imbalance
bool CModel :: ApplyConstraints() { 
	bool bError=false;

	//-- 
	int iState, iConstraint, iConstraints=m_aConstraints.Count();
	bool bBkwd0;
	CModelRate *pRate0;
	double adValue[2];

	//-- all non-loop constraints		
	for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		//-- get constraint
		CModelConstraint * pCon = Constraint(iConstraint);
		if( pCon->m_bError ) 
			continue;

		//-- set constraint flags 
		switch( pCon->m_iType ) { 
			case CONSTRAINT_SCALERATE :
				pCon->m_apAdjust[1]->m_adK0[pCon->m_abAdjustBkwd[1]?1:0] = pCon->m_adValue[0] 
					* pCon->m_apAdjust[0]->m_adK0[pCon->m_abAdjustBkwd[0]?1:0]; 
				break;
				
			case CONSTRAINT_SCALEEXP :
				pCon->m_apAdjust[1]->m_adK1[pCon->m_abAdjustBkwd[1]?1:0] = pCon->m_adValue[1] 
					* pCon->m_apAdjust[0]->m_adK1[pCon->m_abAdjustBkwd[0]?1:0]; 
				break;

			case CONSTRAINT_LOOPBALANCE : 
			case CONSTRAINT_LOOPIMBALANCE : 
				//-- Calc loop product(k0f/k0b) and sum(k1f-k1b)
				adValue[0]=adValue[1]=0.0;
				for( iState=0; iState<pCon->m_iStates-1; ++iState ) {
					pRate0 = FindRate( pCon->m_aiStates[iState], pCon->m_aiStates[iState+1], &bBkwd0 );
					adValue[0] *= pRate0->m_adK0[bBkwd0?1:0] / pRate0->m_adK0[bBkwd0?0:1];
					adValue[1] += pRate0->m_adK1[bBkwd0?1:0] - pRate0->m_adK1[bBkwd0?0:1];
					}
				//-- apply k0 balance 
				if( pCon->m_apAdjust[0] != NULL && fabs( adValue[0]-pCon->m_adValue[0] ) > 1.0e-5 && adValue[0]!=0.0 )
					pCon->m_apAdjust[0]->m_adK0[pCon->m_abAdjustBkwd[0]?1:0] *= pCon->m_adValue[0] / adValue[0];
					
				//-- apply k1 balance 
				if( pCon->m_apAdjust[1] != NULL && fabs( adValue[1]-pCon->m_adValue[1] ) > 1.0e-5 )
					pCon->m_apAdjust[1]->m_adK1[pCon->m_abAdjustBkwd[1]?1:0] += (pCon->m_adValue[1]-adValue[0]);
				break;
			}
		}


	return bError;
	}
	
	
/*	
bool CModel :: ApplyConstraints() { 
	CRateList oBalanced;			// list of rates already balanced by a prior loop constraint
	CModelRate * pRate;
	CModelRate * p1stFwd, * p1stBkwd;	// 1st unprotected forward or backward rates are used to balance loop if needed.
	bool bBkwd;
	int iSt0, iSt1, iState;
	double dFwd, dBkwd;				// product of rates in loop (actually sum of logs).
	int iConType;

	int iConstraint, iConstraints=m_aConstraints.Count();
	for( iConstraint=0; iConstraint<iConstraints; ++iConstraint ) { 
		CModelConstraint * pCon = Constraint(iConstraint);

		if( pCon->m_iType==CONSTRAINT_LOOPBALANCE ) {
			dFwd = dBkwd = 0.0;
			p1stFwd = p1stBkwd = NULL;
			for( iState=0; iState<pCon->m_iStates-1; ++iState ) {
				iSt0 = pCon->m_aiStates[iState];
				iSt1 = pCon->m_aiStates[iState+1];
				//-- ensure rate exists 
				pRate = FindRate( iSt0, iSt1, &bBkwd );
				if( pRate==NULL ) {
					dFwd = dBkwd = 0.0;
					break;
					}
				//-- accumulate rates 
				dFwd += log(pRate->m_adK0[bBkwd?1:0]);
				dBkwd += log(pRate->m_adK0[bBkwd?0:1]);
				//-- determine which rate to adjust if neccessary
				if( ! oBalanced.HasRate( iSt0, iSt1 ) ) { 
					if( p1stFwd==NULL && ! ConstrainedLoopRate( true, iSt0, iSt1 ) ) { 
						iConType = ConstrainedRate( iSt0, iSt1 );
						if( iConType>0 ) 
							p1stFwd=pRate; 
						else if( iConType<0 )
							p1stBkwd=pRate; //-- remember 1st unprotected bkwd rate 
						}
					oBalanced.AddRate( iSt0, iSt1 );	//-- add to already balanced rate list 
					}

				}
			
			//-- adjust 1 rate to enforce loop balance
			if( dBkwd!=dFwd  ) {
				if( p1stFwd!=NULL ) 
					p1stFwd->m_adK0[0]*=exp( dBkwd-dFwd );
				else if( p1stBkwd!=NULL )
					p1stBkwd->m_adK0[1]*=exp( dFwd-dBkwd );
				}
			}
		}

	return true;
	}
*/
	
//-----------------------------------------------------------------------------
// Create the Q matrix.  
Array2D<double> CModel :: Q( CStimuli * poStim, double t ) { 
	//NOTE : poStim may be NULL ! 
	int iStates = m_aStates.Count();	//-- States in model 
	Array2D<double> aQ( iStates, iStates, 0.0 );	// Q matrix to be generated
	
	int iPaths = m_aRates.Count();					//-- paths in model, each has 2 directions
	for( int iPath=0; iPath<iPaths; ++iPath ) {		//-- iterate through rates
		CModelRate * pRate = Rate(iPath);			//-- one fwd / bwd rate
		int iSt0 = pRate->m_aiStates[0];			//-- states connected by rate 
		int iSt1 = pRate->m_aiStates[1];
		aQ[iSt0][iSt0] -= ( aQ[iSt0][iSt1] = pRate->RateValue( true, poStim, t ) );	//-- Forward rate 		
		aQ[iSt1][iSt1] -= ( aQ[iSt1][iSt0] = pRate->RateValue( false, poStim, t ) );//-- Backward rate 		
		}

	return aQ;
	}

//---------------------------------------------------------------------------------------
// returns 0:success;  -1:does not satisfy initial constraints; -2:error in SVD
//		-3:error in SVD.U;  -4: U not orthogonal;  -5: V not orthogonal cols;
//		 -6: V not orthogonal rows;
int CModel :: FreePar( int iConstraintRows, Array1D<double> & aX, Array2D<double> & aMtx, 
					 Array1D<double> & aVct, Array1D<double> & aZ ) {
	const double TOL = 1.0e-5;
	const double VERY_SMALL = 1.0e-9;

	double dSum;
	int iRow, iCol, i, j;
	int iDim = aX.dim();
	
	//-- if no constraints, set array to identity and exit
	if( iConstraintRows==0 ) { 
		aZ = Array1D<double>( iDim );
		aMtx = Array2D<double>( iDim, iDim, 0.0 );
		aVct = Array1D<double>( iDim, 0.0 );
		for( i=0; i<iDim; ++i ) {
			aMtx[i][i]=1.0;
			aZ[i]=aX[i];
			}
		return 0;
		}

	//-- Check if intial values satisfy constraints, return -1 if not.
	for( i=0; i<iConstraintRows; ++i ) {
		dSum = -aVct[i];
		for( j=0; j<iDim; ++j ) 
			dSum+=aMtx[i][j]*aX[j];
		if( fabs(dSum)>1.0e-2 )
			return -1;
		}
		
	//-- SVD
	JAMA::SVD<double> aSVD( aMtx );		// results in aSVD.U,V,S. Use getU,getV,getSingularValues to access.
	Array2D<double> U;
	Array2D<double> V;
	Array1D<double> S;
	aSVD.getU( U );
	aSVD.getV( V );
	aSVD.getSingularValues( S );

	//-- SVD result is sorted, so [0] is the maximum.. Ensure all are >= TOL * maximum.
	double dMax = fabs( S[0] );
	for( i=1; i<iConstraintRows; ++i ) 
		if( S[i] < dMax*TOL ) 
			return -2;

	//-- check that right section of U is small values
	//for( iRow=0; iRow<iConstraints; iRow++ )
	//	for( iCol=iConstraints; iCol<iDim; iCol++ )
	//		if( fabs(U[iRow][iCol])>TOL )
	//			return -3;
				
	//-- Check that U's columns[ 0..iConstraints ] are orthogonal unit vectors
	for( iRow=0; iRow<iConstraintRows; iRow++ )
		for( iCol=0; iCol<iConstraintRows; iCol++ ) { 
			dSum=0.0;
			for( i=0; i<iConstraintRows; i++ ) 
				dSum = dSum + U[i][iRow]*U[i][iCol] ;
			if( fabs( iRow==iCol ? dSum-1.0 : dSum ) > TOL ) 
				return -4;
			}
			
	//-- Check that V's columns and rows are orthogonal unit vectors
	for( iRow=0; iRow<iDim; iRow++ )
		for( iCol=0; iCol<iDim; iCol++ ) { 
			dSum=0.0;
			for( i=0; i<iDim; i++) 
				dSum = dSum + V[i][iRow]*V[i][iCol] ;
			if( fabs( iRow==iCol ? dSum-1.0 : dSum ) > TOL ) 
				return -4;
			}

	for( iRow=0; iRow<iDim; iRow++ )
		for( iCol=0; iCol<iDim; iCol++ ) { 
			dSum=0.0;
			for( i=0; i<iDim; i++) 
				dSum = dSum + V[iRow][i]*V[iCol][i] ;
			if( fabs( iRow==iCol ? dSum-1.0 : dSum ) > TOL ) 
				return -5;
			}
	
	//-- set h for Mtx and vct 
	Array1D<double> h( iConstraintRows );
	matmult( iConstraintRows, iConstraintRows, (double **) U, (double *)aVct, (double *)h );
	for( iRow=0; iRow<iConstraintRows; iRow++ ) 
		h[iRow]=h[iRow]/S[iRow]; 
	
	//-- Create new Mtx and Vct
	aMtx = Array2D<double>( iDim, iDim-iConstraintRows, 0.0 );
	aVct = Array1D<double>( iDim, 0.0 );
	matmult( iDim, iConstraintRows, (double **)V, (double *)h, (double *)aVct );
	for( iRow=0; iRow<iDim; iRow++ )
		for( iCol=iConstraintRows; iCol<iDim; iCol++ )
			if( fabs(V[iRow][iCol]) >= VERY_SMALL ) 
				aMtx[iRow][iCol-iConstraintRows]=V[iRow][iCol];
		
	//-- get z from x 
	aZ = Array1D<double>( iDim-iConstraintRows );
	matmult( iDim, iDim-iConstraintRows, (double *)aX, (double **)aMtx, (double *)aZ );
		
	return 0;
	}
		
//-----------------------------------------------------------------------------
CMetaModelState :: CMetaModelState( int iStates, int * aiStates, double dStartProb ) { 
	m_aiStates = new int[iStates];
	for( int i=0; i<iStates; ++i ) 
		m_aiStates[i] = aiStates[i];
	m_dStartProb = dStartProb;
	m_iClass = -1;
	m_iGroup = -1;
	m_dXPos = m_dYPos = 0.0;
	}

CMetaModelState :: ~CMetaModelState() { 
	delete [] m_aiStates;
	}

//-----------------------------------------------------------------------------
CMetaModelPath :: CMetaModelPath( int i, int j, int iRateIndex, CModelRate * pRate, bool bBwd ) { 
	m_aiStates[0]=i;	
	m_aiStates[1]=j;	
	m_iOriginalIndex = iRateIndex;
	m_pOriginalRate = pRate;
	m_bOriginalBwd = bBwd;
	}

double CMetaModelPath :: RateValueK0( bool bFwd, CStimuli * poStim, double t ){
	return m_pOriginalRate->RateValueK0( bFwd, poStim, t );
	}

double CMetaModelPath :: RateValueK1( bool bFwd, CStimuli * poStim, double t ){
	return m_pOriginalRate->RateValueK1( bFwd, poStim, t );
	}

double CMetaModelPath :: RateValue( bool bFwd, CStimuli * poStim, double t ){
	return m_pOriginalRate->RateValue( bFwd, poStim, t );
	}

//-----------------------------------------------------------------------------
CMetaModelState * CMetaModel :: MetaState( int i ) { 
	return (CMetaModelState *)m_aStates[i];
	}

CMetaModelPath * CMetaModel :: MetaPath( int i ) { 
	return (CMetaModelPath *) m_aPaths[i] ;
	}
	
//-----------------------------------------------------------------------------
CMetaModel :: CMetaModel( ) { 
	}

CMetaModel :: CMetaModel( CModel * pModel, int iChannels, bool bStd ) 
	: CModel( *pModel )  { 
	Setup( pModel, iChannels, bStd );
	}
	
void CMetaModel :: Setup( CModel * pModel, int iChannels, bool bStd ) { 
	//-- get state count from model and calculate metastate count
	int i, j;
	int iMetaState=0;
	int iStates=pModel->m_aStates.Count();	
	if( iStates==0 || iChannels<1 ) 
		return;

	//-- build array of model start probs and scale so sum()=1.0
	double * adStartProb = new double[iStates];
	pModel->NormalizeStartProbabilities(adStartProb);

	//-- build metastates - array of channels in each state and their conductance adMetaAmp
	int iMetaStates = BinCoeffInt( iChannels+iStates-1, iChannels ); // calc metstate count
	double dStartProb;
	double dAmp0 = pModel->Class(0)->m_dAmp;
	double dStd0 = pModel->Class(0)->m_dStdDev;
	bool bOpenPositive = ( pModel->Class(1)->m_dAmp > dAmp0 );
	double * adMetaAmp = new double[iMetaStates];	// amp array 
	double * adMetaStd = NULL;						// std array 
	//if( bStd ) 
		adMetaStd = new double[iMetaStates];	
	CCombState o( iChannels, iStates );

	m_aStates.DeleteAll();
	do {
		dStartProb=LogFactorial(iChannels);	
		adMetaAmp[iMetaState]=dAmp0;	
		adMetaStd[iMetaState]=dStd0;	
		for( int iState=0; iState<iStates; ++iState ) {
			//-- calc amp and Std
			adMetaAmp[iMetaState] += o.m_ai[iState] * (pModel->StateAmp(iState)-dAmp0);
			if( bStd ) { 
				double dStdIncr = pModel->StateStd(iState)*pModel->StateStd(iState)-dStd0*dStd0;
				adMetaStd[iMetaState] = sqrt( adMetaStd[iMetaState]*adMetaStd[iMetaState] + o.m_ai[iState] * dStdIncr ); 
				}
			else 
				adMetaStd[iMetaState]=0.0;
			//-- calc start prob
			dStartProb += o.m_ai[iState] * log(max(adStartProb[iState],1.0e-10)) - LogFactorial(o.m_ai[iState]);
			}
		iMetaState++;
		
		m_aStates.AddItem( new CMetaModelState( iStates, o.m_ai, exp(dStartProb) ) );	
		} while( o.NextCombination() );
		
	delete [] adStartProb;

	//-- repeatedly find minimum or maximum and all within 1.0e-5 of it, condense (formerly eclass())
	m_aClasses.DeleteAll();
	int iPos;
	while( (iPos=FindExtremeAmp( bOpenPositive, adMetaAmp, iMetaState )) != -1 )
		SetExtremeAmp( adMetaAmp[iPos], adMetaStd[iPos], iMetaStates, adMetaAmp, adMetaStd );
	delete [] adMetaAmp;
	if( bStd ) 
		delete [] adMetaStd;
	int iClasses = m_aClasses.Count();
		
	//-- find metastates which differ by state of 1 channel only and are
	//-- connected in the original model and add metapaths.
	int iMetaPath=0, iMetaPaths=0, iFrom, iTo;
	int iRate, aiStates[4];
	bool bBkwd;
	double dXPos=90.0, dYPos=80.0;
	CMetaModelPath * pPath;
	m_aPaths.DeleteAll();
	TNT::Array2D<short>aiMap( iStates, iStates, -1 ); 
	for( i=0; i+1<iMetaStates; ++i ) 
		for( j=i+1; j<iMetaStates; ++j )
			if( MetaConnect(i,j,iStates,&iFrom,&iTo) && (iRate=pModel->FindRateIndex(iFrom,iTo,&bBkwd))!=-1 ) { 
				//-- add path 			
				m_aPaths.AddItem( new CMetaModelPath( i, j, iRate, pModel->Rate(iRate), bBkwd ) );
				
				//-- add constraint 
				if( iFrom==0 || iTo==0 ) { 
					if( aiMap[iFrom][iTo]==-1 ) { 
						aiMap[iFrom][iTo]=m_aPaths.Count()-1;
						}
					else { 
						if( dYPos<20 ) {
							dXPos=max(20.0, dXPos-CONSTR_XOFFSET);
							dYPos=80.0;
							}
						else 
							dYPos-=CONSTR_YOFFSET*2;
						pPath = MetaPath( aiMap[iFrom][iTo] );
						aiStates[0] = pPath->m_aiStates[0];
						aiStates[1] = pPath->m_aiStates[1];
						aiStates[2] = i;
						aiStates[3] = j;
						AddConstraint( CONSTRAINT_SCALERATE, 4, aiStates, dXPos, dYPos );
						aiStates[1] = pPath->m_aiStates[0];
						aiStates[0] = pPath->m_aiStates[1];
						aiStates[3] = i;
						aiStates[2] = j;
						AddConstraint( CONSTRAINT_SCALERATE, 4, aiStates, dXPos, dYPos-CONSTR_YOFFSET );
						}
					}
				}
				
	}

int CMetaModel :: FindExtremeAmp( bool bPositive, double * adMetaAmp, int iAmps  ) { 
	CMetaModelState * pState;
	int iPos=-1;
	double dAmp;
	for( int i=0; i<iAmps; ++i ) { 
		pState = MetaState(i);
		if( pState->m_iClass==-1 ) 
			if( (iPos==-1) || (bPositive==(dAmp>adMetaAmp[i])) ) { 
				dAmp = adMetaAmp[i];
				iPos=i;
				}
		}
	return iPos;
	}

void CMetaModel :: SetExtremeAmp( double dAmp, double dStd, int iMetaStates, double * adMetaAmp, double * adMetaStd, double dTol ) { 
	int iClass=m_aClasses.Count();
	m_aClasses.AddItem( new CModelClass( dAmp, dStd ) );
	for( int i=0; i<iMetaStates; ++i ) 
		if( MetaState(i)->m_iClass==-1 && fabs(adMetaAmp[i]-dAmp)<dTol && (adMetaStd==NULL || fabs(adMetaStd[i]-dStd)<dTol) )
			MetaState(i)->m_iClass=iClass;
	}

bool CMetaModel :: MetaConnect( int i, int j, int iStates, int * piFrom, int * piTo ) { 
	// find two metastates where differences in channel count contain only 0's, a single 1 and a single -1.
	*piFrom=-1;
	*piTo=-1;
	int * ai0 = MetaState(i)->m_aiStates;
	int * ai1 = MetaState(j)->m_aiStates;

	for( int i=0; i<iStates; ++i ) {
		int iDiff=ai0[i]-ai1[i];
		if( iDiff==1 ) {
			if( *piFrom!=-1 ) 
				return false;
			*piFrom=i;
			}
		else if( iDiff==-1 ) {
			if( *piTo!=-1 ) 
				return false;
			*piTo=i;
			}
		else if( iDiff != 0 ) 
			return false;
		}
	return *piFrom!=-1 && *piTo!=-1;
	}

//-----------------------------------------------------------------------------
// Create the Q matrix.  
// TODO - provide the stimulus array
Array2D<double> CMetaModel :: QQ( CStimuli * poStim, double t, Array2D<double> * padQQ_q ) { 
	int iStates = m_aStates.Count();
	Array2D<double> aQ( iStates, iStates, 0.0 );
	if( padQQ_q!=NULL ) 
		*padQQ_q=0.0;
	
	int iChannels, iPaths = m_aPaths.Count();	
	int iSt0, iSt1;
	
	for( int iPath=0; iPath<iPaths; ++iPath ) {				
		CMetaModelPath * pMetaPath = MetaPath(iPath);		
		iSt0 = pMetaPath->m_aiStates[0];					
		iSt1 = pMetaPath->m_aiStates[1];					
		
		//-- rate from source to dest.  // C[idrug[k]] * exp( x[2*k] + V[ivolt[k]] * x[2*k+1] );
		aQ[iSt0][iSt0] -= (	aQ[iSt0][iSt1] = RateValue( pMetaPath, true, poStim, t, &iChannels ) );			

		if( padQQ_q!=NULL ) 
			(*padQQ_q)[iPath*2][pMetaPath->m_iOriginalIndex*2+0] = iChannels;

		//-- rate from dest to source.
		aQ[iSt1][iSt1] -= ( aQ[iSt1][iSt0] = RateValue( pMetaPath, false, poStim, t, &iChannels ) ); 
		
		if( padQQ_q!=NULL ) 
			(*padQQ_q)[iPath*2+1][pMetaPath->m_iOriginalIndex*2+1] = iChannels;
		}

	return aQ;
	}

void CMetaModel :: SetupClassStateIndex( int iUsedClasses, int iStates ) { 
	int iClass=0, iNotInClass;

	//-----  - { states in each class , states not in each class } 
	m_aiClassStateCount = Array1D<int>( iUsedClasses, 0 );
	m_aiClassStateIndex = Array2D<int>( iUsedClasses, iStates*2, -1 );
	for( int i=0; i<m_aClasses.Count(); i++ ) {
		if( UsesClass(i) ) { 
			for( int iState=0; iState<iStates; ++iState ) {
				if( i==MetaState(iState)->m_iClass ) {
					m_aiClassStateIndex[iClass][m_aiClassStateCount[iClass]]=iState;
					m_aiClassStateCount[iClass]=m_aiClassStateCount[iClass]+1;
					}
				else {
					iNotInClass=iState-m_aiClassStateCount[iClass];
					m_aiClassStateIndex[iClass][iStates+iNotInClass]=iState;
					}
				}
			++iClass;
			}
		}
	}
	
double CMetaModel :: RateValueK0( CMetaModelPath * pMetaPath, bool bFwd, CStimuli * poStim, double t, int * piChannels){ 
	double nK0 = pMetaPath->RateValueK0( bFwd, poStim, t );
	int iMetaSrc = pMetaPath->m_aiStates[bFwd?0:1];
	int iOrigSrc = pMetaPath->m_pOriginalRate->m_aiStates[pMetaPath->m_bOriginalBwd==bFwd?1:0];
	int iChannels = MetaState( iMetaSrc )->m_aiStates[iOrigSrc]; 
	if( piChannels != NULL ) 
		*piChannels=iChannels;
	return nK0 * iChannels;			
	}
	
double CMetaModel :: RateValueK1( CMetaModelPath * pMetaPath, bool bFwd, CStimuli * poStim, double t){ 
	return pMetaPath->RateValueK1( bFwd, poStim, t );
	}
	
double CMetaModel :: RateValue( CMetaModelPath * pMetaPath, bool bFwd, CStimuli * poStim, double t, int * piChannels ) { 
	return RateValueK0( pMetaPath, bFwd, poStim, t, piChannels ) 
		* exp( RateValueK1( pMetaPath, bFwd, poStim, t ) );
	}
	
void CMetaModel :: FillPathArray( Array2D<int> & ai1Paths, CStimuli * aoStim, double t  ){ 
	int iFill=0;
	int i1Paths=ai1Paths.dim1();
	int iPaths=m_aPaths.Count();
	for( int iPath=0; iPath<iPaths && iFill<i1Paths; ++iPath ) {
		CMetaModelPath * pPath = MetaPath( iPath );
		ai1Paths[iFill][0]=pPath->m_aiStates[0];
		ai1Paths[iFill][1]=pPath->m_aiStates[1];
		iFill++;
		ai1Paths[iFill][0]=pPath->m_aiStates[1];
		ai1Paths[iFill][1]=pPath->m_aiStates[0];
		iFill++;
		}
	}

//-- Fill in a rate object with calculated values for specified path in metamodel
bool CMetaModel :: MakeRate( int iPath, CModelRate * pRate ){ 
	CMetaModelPath * pPath = MetaPath( iPath );	// pointer to path in metamodel
	if( pPath==NULL ) 
		return false;
	CModelRate * pRef = pPath->m_pOriginalRate;	// ptr to rate in original model
	int i0 = pPath->m_bOriginalBwd ? 1 : 0;
		
	//-- get metastates connected by pPath
	//-- state indices in metamodel are indices in generated rate 
	CMetaModelState * pStates[2];
	pStates[0] = MetaState( pRate->m_aiStates[0]=pPath->m_aiStates[0] );
	pStates[1] = MetaState( pRate->m_aiStates[1]=pPath->m_aiStates[1] );
		
	//-- fill in generated rate parameters
	int iFromOrig = pRef->m_aiStates[i0];
	pRate->m_adK0[0] = pRef->m_adK0[i0] * pStates[0]->m_aiStates[iFromOrig];
	pRate->m_adK1[0] = pRef->m_adK1[i0];
	pRate->m_abPDep[0] = pRef->m_abPDep[i0];
	pRate->m_acPName[0] = pRef->m_acPName[i0];
	pRate->m_abQDep[0] = pRef->m_abQDep[i0];
	pRate->m_acQName[0] = pRef->m_acQName[i0];

	iFromOrig = pRef->m_aiStates[1-i0];
	pRate->m_adK0[1] = pRef->m_adK0[1-i0] * pStates[1]->m_aiStates[iFromOrig];
	pRate->m_adK1[1] = pRef->m_adK1[1-i0];
	pRate->m_abPDep[1] = pRef->m_abPDep[1-i0];
	pRate->m_acPName[1] = pRef->m_acPName[1-i0];
	pRate->m_abQDep[1] = pRef->m_abQDep[1-i0];
	pRate->m_acQName[1] = pRef->m_acQName[1-i0];
	return true;
	}
				
